@@ -19,12 +19,25 @@ import           Data.Generics
 #if __GLASGOW_HASKELL__ < 707
 import           GHC hiding (flags, Module, Located)
 import           MonadUtils (liftIO, MonadIO)
-#else
+#elif __GLASGOW_HASKELL__ < 900
 import           GHC hiding (Module, Located)
 import           DynFlags
 import           MonadUtils (liftIO)
+#else
+import           GHC hiding (Module, Located)
+import           GHC.Driver.Session
+import           GHC.Utils.Monad (liftIO)
 #endif
+
+#if __GLASGOW_HASKELL__ < 900
+import           Digraph (flattenSCCs)
 import           Exception (ExceptionMonad)
+#else
+import           GHC.Data.Graph.Directed (flattenSCCs)
+import           GHC.Utils.Exception (ExceptionMonad)
+import           Control.Monad.Catch (generalBracket)
+#endif
+
 import           System.Directory
 import           System.FilePath
 
@@ -37,8 +50,6 @@ import           Coercion (Coercion)
 import           FastString (unpackFS)
 #endif
 
-import           Digraph (flattenSCCs)
-
 import           System.Posix.Internals (c_getpid)
 
 import           GhcUtil (withGhc)
@@ -48,7 +59,15 @@ import           Util (convertDosLineEndings)
 import           PackageDBs (getPackageDBArgs)
 
 #if __GLASGOW_HASKELL__ >= 806
+#if __GLASGOW_HASKELL__ < 900
 import           DynamicLoading (initializePlugins)
+#else
+import           GHC.Runtime.Loader (initializePlugins)
+#endif
+#endif
+
+#if __GLASGOW_HASKELL__ >= 901
+import           GHC.Unit.Module.Graph
 #endif
 
 -- | A wrapper around `SomeException`, to allow for a custom `Show` instance.
@@ -85,12 +104,6 @@ instance NFData a => NFData (Module a) where
 
 #if __GLASGOW_HASKELL__ < 803
 type GhcPs = RdrName
-
-needsTemplateHaskellOrQQ :: ModuleGraph -> Bool
-needsTemplateHaskellOrQQ = needsTemplateHaskell
-
-mapMG :: (ModSummary -> ModSummary) -> ModuleGraph -> ModuleGraph
-mapMG = map
 #endif
 
 #if __GLASGOW_HASKELL__ < 805
@@ -99,36 +112,26 @@ addQuoteInclude includes new = new ++ includes
 #endif
 
 -- | Parse a list of modules.
-parse :: [String] -> IO [TypecheckedModule]
+parse :: [String] -> IO [ParsedModule]
 parse args = withGhc args $ \modules_ -> withTempOutputDir $ do
 
   -- ignore additional object files
   let modules = filter (not . isSuffixOf ".o") modules_
 
-  mapM (`guessTarget` Nothing) modules >>= setTargets
+  setTargets =<< forM modules (\ m -> guessTarget m
+#if __GLASGOW_HASKELL__ >= 903
+                Nothing
+#endif
+                Nothing)
   mods <- depanal [] False
 
-  mods' <- if needsTemplateHaskellOrQQ mods then enableCompilation mods else return mods
-
-  let sortedMods = flattenSCCs (topSortModuleGraph False mods' Nothing)
-  reverse <$> mapM (loadModPlugins >=> parseModule >=> typecheckModule >=> loadModule) sortedMods
-  where
-    -- copied from Haddock/Interface.hs
-    enableCompilation :: ModuleGraph -> Ghc ModuleGraph
-    enableCompilation modGraph = do
-#if __GLASGOW_HASKELL__ < 707
-      let enableComp d = d { hscTarget = defaultObjectTarget }
-#elif __GLASGOW_HASKELL__ < 809
-      let enableComp d = let platform = targetPlatform d
-                         in d { hscTarget = defaultObjectTarget platform }
-#else
-      let enableComp d = d { hscTarget = defaultObjectTarget d }
+  let sortedMods = flattenSCCs
+#if __GLASGOW_HASKELL__ >= 901
+                     $ filterToposortToModules
 #endif
-      modifySessionDynFlags enableComp
-      -- We need to update the DynFlags of the ModSummaries as well.
-      let upd m = m { ms_hspp_opts = enableComp (ms_hspp_opts m) }
-      let modGraph' = mapMG upd modGraph
-      return modGraph'
+                     $ topSortModuleGraph False mods Nothing
+  reverse <$> mapM (loadModPlugins >=> parseModule) sortedMods
+  where
 
     -- copied from Haddock/GhcUtils.hs
     modifySessionDynFlags :: (DynFlags -> DynFlags) -> Ghc ()
@@ -159,7 +162,11 @@ parse args = withGhc args $ \modules_ -> withTempOutputDir $ do
     -- | A variant of 'gbracket' where the return value from the first computation
     -- is not required.
     gbracket_ :: ExceptionMonad m => m a -> m b -> m c -> m c
+#if __GLASGOW_HASKELL__ < 900
     gbracket_ before_ after thing = gbracket before_ (const after) (const thing)
+#else
+    gbracket_ before_ after thing = fst <$> generalBracket before_ (\ _ _ -> after) (const thing)
+#endif
 
     setOutputDir f d = d {
         objectDir  = Just f
@@ -171,9 +178,21 @@ parse args = withGhc args $ \modules_ -> withTempOutputDir $ do
 #if __GLASGOW_HASKELL__ >= 806
     -- Since GHC 8.6, plugins are initialized on a per module basis
     loadModPlugins modsum = do
+      _ <- setSessionDynFlags (GHC.ms_hspp_opts modsum)
       hsc_env <- getSession
+
+# if __GLASGOW_HASKELL__ >= 903
+      hsc_env' <- liftIO (initializePlugins hsc_env Nothing)
+      setSession hsc_env'
+      return $ modsum
+# elif __GLASGOW_HASKELL__ >= 901
+      hsc_env' <- liftIO (initializePlugins hsc_env)
+      setSession hsc_env'
+      return $ modsum
+# else
       dynflags' <- liftIO (initializePlugins hsc_env (GHC.ms_hspp_opts modsum))
       return $ modsum { ms_hspp_opts = dynflags' }
+# endif
 #else
     loadModPlugins = return
 #endif
@@ -187,7 +206,7 @@ extract args = do
   packageDBArgs <- getPackageDBArgs
   let args'  = args ++ packageDBArgs
   mods <- parse args'
-  let docs = map (fmap (fmap convertDosLineEndings) . extractFromModule . tm_parsed_module) mods
+  let docs = map (fmap (fmap convertDosLineEndings) . extractFromModule) mods
 
   (docs `deepseq` return docs) `catches` [
       -- Re-throw AsyncException, otherwise execution will not terminate on
@@ -219,13 +238,15 @@ docStringsFromModule mod = map (fmap (toLocated . fmap unpackHDS)) docs
     -- traversing the whole source in a generic way, to ensure that we get
     -- everything in source order.
     header  = [(Nothing, x) | Just x <- [hsmodHaddockModHeader source]]
+    exports = [ (Nothing, L (locA loc) doc)
 #if __GLASGOW_HASKELL__ < 710
-    exports = [(Nothing, L loc doc) | L loc (IEDoc doc) <- concat (hsmodExports source)]
+              | L loc (IEDoc doc) <- concat (hsmodExports source)
 #elif __GLASGOW_HASKELL__ < 805
-    exports = [(Nothing, L loc doc) | L loc (IEDoc doc) <- maybe [] unLoc (hsmodExports source)]
+              | L loc (IEDoc doc) <- maybe [] unLoc (hsmodExports source)
 #else
-    exports = [(Nothing, L loc doc) | L loc (IEDoc _ doc) <- maybe [] unLoc (hsmodExports source)]
+              | L loc (IEDoc _ doc) <- maybe [] unLoc (hsmodExports source)
 #endif
+              ]
     decls   = extractDocStrings (hsmodDecls source)
 
 type Selector a = a -> ([(Maybe String, LHsDocString)], Bool)
@@ -279,15 +300,21 @@ extractDocStrings = everythingBut (++) (([], False) `mkQ` fromLHsDecl
       -- no location information attached.  The location information is
       -- attached to HsDecl instead.
 #if __GLASGOW_HASKELL__ < 805
-      DocD x -> select (fromDocDecl loc x)
+      DocD x
 #else
-      DocD _ x -> select (fromDocDecl loc x)
+      DocD _ x
 #endif
+               -> select (fromDocDecl (locA loc) x)
 
       _ -> (extractDocStrings decl, True)
 
-    fromLDocDecl :: Selector LDocDecl
-    fromLDocDecl (L loc x) = select (fromDocDecl loc x)
+    fromLDocDecl :: Selector
+#if __GLASGOW_HASKELL__ >= 901
+                             (LDocDecl GhcPs)
+#else
+                             LDocDecl
+#endif
+    fromLDocDecl (L loc x) = select (fromDocDecl (locA loc) x)
 
     fromLHsDocString :: Selector LHsDocString
     fromLHsDocString x = select (Nothing, x)
@@ -302,3 +329,8 @@ extractDocStrings = everythingBut (++) (([], False) `mkQ` fromLHsDecl
 unpackHDS :: HsDocString -> String
 unpackHDS (HsDocString s) = unpackFS s
 #endif
+
+#if __GLASGOW_HASKELL__ < 901
+locA :: SrcSpan -> SrcSpan
+locA = id
+#endif
