@@ -1,10 +1,18 @@
+{-# LANGUAGE CPP #-}
+
 module LocationSpec (main, spec) where
 
 import           Test.Hspec
 
 import           Location
+
+#if __GLASGOW_HASKELL__ < 900
 import           SrcLoc
 import           FastString (fsLit)
+#else
+import           GHC.Types.SrcLoc
+import           GHC.Data.FastString (fsLit)
+#endif
 
 main :: IO ()
 main = hspec spec
