@@ -8,15 +8,8 @@ import           Test.Hspec
 import           System.Exit
 
 import qualified Control.Exception as E
-#if __GLASGOW_HASKELL__ < 707
-import           System.Cmd
-#else
-import           System.Process
-#endif
-import           System.Directory (getCurrentDirectory, setCurrentDirectory, removeDirectoryRecursive)
-import           Data.List.Compat
-
-import           System.Environment.Compat
+import           System.Directory (getCurrentDirectory, setCurrentDirectory)
+import           Data.List.Compat (isPrefixOf, sort)
 
 import           System.IO.Silently
 import           System.IO (stderr)
@@ -33,16 +26,6 @@ withCurrentDirectory workingDir action = do
     setCurrentDirectory workingDir
     action
 
-rmDir :: FilePath -> IO ()
-rmDir dir = removeDirectoryRecursive dir `E.catch` (const $ return () :: E.IOException -> IO ())
-
-withEnv :: String -> String -> IO a -> IO a
-withEnv k v action = E.bracket save restore $ \_ -> do
-  setEnv k v >> action
-  where
-    save    = lookup k <$> getEnvironment
-    restore = maybe (unsetEnv k) (setEnv k)
-
 main :: IO ()
 main = hspec spec
 
@@ -81,22 +64,6 @@ spec = do
         , "Try `doctest --help' for more information."
         ]
 
-    it "respects HASKELL_PACKAGE_SANDBOX" $ do
-      withCurrentDirectory "test/integration/custom-package-conf/foo" $ do
-        ExitSuccess <- rawSystem "ghc-pkg" ["init", "../packages"]
-        ExitSuccess <- rawSystem "cabal" ["v1-configure", "--disable-optimization", "--disable-library-profiling", "--package-db=../packages"]
-        ExitSuccess <- rawSystem "cabal" ["v1-build"]
-        ExitSuccess <- rawSystem "cabal" ["v1-register", "--inplace"]
-        return ()
-
-      withEnv "HASKELL_PACKAGE_SANDBOX" "test/integration/custom-package-conf/packages" $ do
-        hCapture_ [stderr] (doctest ["test/integration/custom-package-conf/Bar.hs"])
-          `shouldReturn` "Examples: 2  Tried: 2  Errors: 0  Failures: 0\n"
-
-      `E.finally` do
-        rmDir "test/integration/custom-package-conf/packages/"
-        rmDir "test/integration/custom-package-conf/foo/dist/"
-
     it "prints verbose description of a specification" $ do
       (r, ()) <- hCapture [stderr] $ doctest ["--verbose", "test/integration/testSimple/Fib.hs"]
       r `shouldBe` unlines [
